package com.ruoyi.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.domain.ReviewDetailsResult;
import com.ruoyi.service.ReviewDetailsResultService;
import com.ruoyi.mapper.ReviewDetailsResultMapper;
import com.ruoyi.web.request.ReviewDetailsResultSaveRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
* @author wangfei
* @description 针对表【t_review_details_result(审查细则的审查结果)】的数据库操作Service实现
* @createDate 2023-12-15 15:42:40
*/
@Transactional
@Service
public class ReviewDetailsResultServiceImpl extends ServiceImpl<ReviewDetailsResultMapper, ReviewDetailsResult> implements ReviewDetailsResultService {

    @Autowired
    private ReviewDetailsResultMapper reviewDetailsResultMapper;

    @Override
    public List<ReviewDetailsResult> findByTaskId(Long taskId) {
        return reviewDetailsResultMapper.findByTaskId(taskId);
    }

    @Override
    public ReviewDetailsResult findByTaskIdAndDetailsId(Long taskId, Long reviewDetailsId) {
        return reviewDetailsResultMapper.findByTaskIdAndDetailsId(taskId, reviewDetailsId);
    }

    @Override
    public void save(ReviewDetailsResultSaveRequest request) {
        ReviewDetailsResult result = reviewDetailsResultMapper.findByTaskIdAndDetailsId(request.getTaskId(), request.getReviewDetailsId());
        // 获取当前登陆人
        SysUser loginUser = SecurityUtils.getLoginUser().getUser();

        if (result != null) {
            // 更新
            result.setPassed(request.getPassed());
            result.setUserId(loginUser.getUserId());
            result.setUserName(loginUser.getNickName());
            result.setLastModifyTime(new Date());
            result.setCatalogue(request.getCatalogue());
            result.setReviewEnterpriseArchiveId(request.getReviewEnterpriseArchiveId());

            this.baseMapper.updateById(result);
        } else {
            // 新增
            ReviewDetailsResult reviewDetailsResult = new ReviewDetailsResult();
            reviewDetailsResult.setTaskId(request.getTaskId());
            reviewDetailsResult.setReviewDetailsId(request.getReviewDetailsId());
            reviewDetailsResult.setReviewEnterpriseArchiveId(request.getReviewEnterpriseArchiveId());
            reviewDetailsResult.setCatalogue(request.getCatalogue());
            reviewDetailsResult.setPassed(request.getPassed());
            reviewDetailsResult.setUserId(loginUser.getUserId());
            reviewDetailsResult.setUserName(loginUser.getNickName());
            reviewDetailsResult.setLastModifyTime(new Date());

            this.baseMapper.insert(reviewDetailsResult);
        }
    }
}




