package com.ruoyi.service.impl;

import com.ruoyi.common.DataStatisticsUtils;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.domain.*;
import com.ruoyi.mapper.*;
import com.ruoyi.service.*;
import com.ruoyi.web.response.CarReviewTaskViewResponse;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.ruoyi.common.DataStatisticsUtils.calculateStandardsPass;

@Transactional
@Service
public class StrategyCarReviewTaskPending implements StrategyCarReviewTask, InitializingBean {

    @Autowired
    private StrategyCarReviewTaskContext strategyCarReviewTaskContext;
    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private ReviewStandardService reviewStandardService;

    @Autowired
    private ReviewDetailsResultService reviewDetailsResultService;

    @Autowired
    private ReviewSceneChangeTaskService reviewSceneChangeTaskService;

    @Autowired
    private CarReviewTaskMapper carReviewTaskMapper;

    @Autowired
    private ReviewDetailsResultMapper reviewDetailsResultMapper;

    @Autowired
    private ReviewDetailsMapper reviewDetailsMapper;

    @Autowired
    private ReviewTaskInboxMapper reviewTaskInboxMapper;

    @Autowired
    private TaskMapper taskMapper;

    @Autowired
    private TaskSampleRelationMapper taskSampleRelationMapper;

    @Autowired
    private SampleManagementMapper sampleManagementMapper;

    @Autowired
    private ReviewEnterpriseArchiveMapper reviewEnterpriseArchiveMapper;

    @Autowired
    private ReviewKeyPointService reviewKeyPointService;

    @Autowired
    private ReviewDetailsService reviewDetailsService;

    @Autowired
    private ReviewStandardMapper reviewStandardMapper;

    @Autowired
    private TaskUserRelationMapper taskUserRelationMapper;

    // PENDING状态下
    // 1.standard字段查询当前最新问卷
    // 2.查询场景变更任务
    // 3.需要联查问卷答案
    @Override
    public CarReviewTaskViewResponse doView(CarReviewTask carReviewTask) {
        // 获取最新问卷
        List<ReviewStandard> standardTree = reviewStandardService.findTree(carReviewTask.getStandardId(), ReviewStandard.TYPE_CAR);

        // 获取当前问卷答案
        List<ReviewDetailsResult> results = reviewDetailsResultService.findByTaskId(carReviewTask.getId());

        // 任务样品关系
        List<TaskSampleRelation> relations = taskSampleRelationMapper.selectByTaskId(taskMapper.findByCarReviewTaskId(carReviewTask.getId()),TaskSampleRelation.COMPLETE_VEHICLE_SAMPLE);

        if (results.size() != 0 && results != null) {
            for (ReviewDetailsResult reviewDetailsResult : results) {
                if (reviewDetailsResult != null && reviewDetailsResult.getReviewEnterpriseArchiveId() == null && relations.size() != 0 && relations != null) {
                    TaskSampleRelation taskSampleRelation = relations.get(0);
                    Sample sample = sampleManagementMapper.selectSampleManagementById(taskSampleRelation.getSampleId());
                    reviewDetailsResult.setEnterpriseName(sample.getManufacturingEnterprise());
                }
            }
        }

        // 绑定问卷与答案
        setDetailsResult(standardTree, results,carReviewTask);

        // 赋值给任务
        carReviewTask.setStandard(standardTree);

        // 查询场景变更任务便于提醒用户场景变动
        List<ReviewSceneChangeTask> lastChanges = reviewSceneChangeTaskService.findLastChangeByTaskId(carReviewTask.getId());

        // 构建返回结果

        CarReviewTaskViewResponse response = new CarReviewTaskViewResponse();
        response.setCarReviewTask(carReviewTask);
        response.setReviewSceneChangeTasks(lastChanges);

        return response;
    }

    @Override
    public void doStartReview(CarReviewTask carReviewTask) {
        throw new ServiceException("不能开始一个进行中的任务", HttpStatus.ERROR);
    }

    @Override
    public void doConfirmReview(CarReviewTask carReviewTask) {
        List<ReviewDetails> detailsList = reviewDetailsMapper.findNoResult(carReviewTask.getId(),CarReviewTask.TYPE);
        List<ReviewTaskInbox> inboxList = reviewTaskInboxMapper.findConfirmListByTaskId(carReviewTask.getId());
        if (detailsList.size() != 0 && detailsList != null) {
            throw new ServiceException("存在未填写的问卷，请填写完整", HttpStatus.ERROR, detailsList);
        } else if (inboxList.size() != 0 && inboxList != null){
            throw new ServiceException("存在未确认的场景变更信息，请确认", HttpStatus.ERROR);
        }else {
            // 1. 获取最新问卷
            List<ReviewStandard> standardTree = reviewStandardService.findTree(carReviewTask.getStandardId(), ReviewStandard.TYPE_CAR);

            // 2. 所有细则必须有对应答案
            List<ReviewDetailsResult> results = reviewDetailsResultService.findByTaskId(carReviewTask.getId());

            // 3. 将最终问卷归档到 standard 字段
            setDetailsResult(standardTree, results,carReviewTask);
            carReviewTask.setStandard(standardTree);

            // 4. 修改任务状态为 SIGNED 待签字
            carReviewTask.setTaskStatus(CarReviewTask.TASK_STATUS_SIGNED);

            // 7. 设置问卷结果
            if (reviewDetailsResultMapper.findNotPass(carReviewTask.getId()) > 0) {
                carReviewTask.setTaskResult("REJECT");
            } else {
                carReviewTask.setTaskResult("PASS");
            }
            carReviewTaskMapper.updateById(carReviewTask);
        }
    }

    @Override
    public void doSubmitReview(CarReviewTask carReviewTask,List<String> imagesUrl) {

        throw new ServiceException("不能提交一个未确认的任务", HttpStatus.ERROR);

    }

    private void setDetailsResult(List<ReviewStandard> standards, List<ReviewDetailsResult> results, CarReviewTask carReviewTask) {

        Map<Long, ReviewDetailsResult> resultMap = results.stream().collect(Collectors.toMap(ReviewDetailsResult::getReviewDetailsId, Function.identity(), (v1, v2) -> v1));

        for(ReviewStandard standard : standards) {
            standard.setPassed(ReviewStandard.PASSED);
            List<ReviewKeyPoint> keyPointList = standard.getKeyPointList();

            for(ReviewKeyPoint keypoint : keyPointList) {
                List<ReviewDetails> reviewDetailsList = keypoint.getReviewDetailsList();

                for(ReviewDetails details : reviewDetailsList) {
                    ReviewDetailsResult reviewDetailsResult = resultMap.get(details.getId());
                    if(reviewDetailsResult == null || Objects.equals(reviewDetailsResult.getPassed(), ReviewStandard.NO_PASSED)) {
                        standard.setPassed(ReviewStandard.NO_PASSED);
                    }
                    if(reviewDetailsResult != null && reviewDetailsResult.getReviewEnterpriseArchiveId() != null) {
                        ReviewEnterpriseArchive reviewEnterpriseArchive = reviewEnterpriseArchiveMapper.selectById(reviewDetailsResult.getReviewEnterpriseArchiveId());
                        reviewDetailsResult.setEnterpriseName(reviewEnterpriseArchive.getEnterpriseName());
                    }
                    details.setResult(reviewDetailsResult);
                }
            }
        }


        List<TaskUserRelation> relationList = taskUserRelationMapper.selectQTeamMembers(taskMapper.findByCarReviewTaskId(carReviewTask.getId()));

        carReviewTask.setStandards(DataStatisticsUtils.calculateStandards(standards));
        BigDecimal standardPass = new BigDecimal(calculateStandardsPass(standards, resultMap) / (double) carReviewTask.getStandards());
        carReviewTask.setStandardsPass(standardPass.setScale(3,BigDecimal.ROUND_UP).doubleValue() * 100);
        carReviewTask.setDetails(DataStatisticsUtils.calculateDetails(standards));
        BigDecimal detailsPass = new BigDecimal(DataStatisticsUtils.calculateDetailsPass(standards, resultMap) / (double) carReviewTask.getDetails());
        carReviewTask.setDetailsPass(detailsPass.setScale(3,BigDecimal.ROUND_UP).doubleValue() * 100);
        carReviewTask.setPerson(relationList.size());

    }


    @Override
    public void afterPropertiesSet() throws Exception {
        strategyCarReviewTaskContext.putResource(CarReviewTask.TASK_STATUS_PENDING, applicationContext.getBean(this.getClass()));
    }
}
