package com.ruoyi.service.impl;

import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.domain.CarReviewTask;
import com.ruoyi.domain.SystemReviewTask;
import com.ruoyi.domain.Task;
import com.ruoyi.mapper.ReviewDetailsMapper;

import com.ruoyi.service.TaskService;
import com.ruoyi.service.TaskState;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Transactional
@Service
public class TaskStatePending extends TaskStateAdapter implements TaskState, InitializingBean {

    @Autowired
    private TaskStateContext taskStateContext;

    @Autowired
    private ReviewDetailsMapper reviewDetailsMapper;

    @Autowired
    private TaskService taskService;

    @Autowired
    private ApplicationContext applicationContext;



    @Override
    public void doCreate(Task task, List<String> taskList) {
        throw new ServiceException("不能创建一个已经开始的任务", HttpStatus.ERROR);
    }

    @Override
    public void doSubmit(Task task) {

        long subTaskCount = 0;

        if(task.getSystemReviewTaskId() != null) {
            subTaskCount = reviewDetailsMapper.count(task.getSystemReviewTaskId(), SystemReviewTask.TYPE);
        }
        if(task.getCarReviewTaskId() != null) {
            subTaskCount = reviewDetailsMapper.count(task.getCarReviewTaskId(), CarReviewTask.TYPE);
        }

        if(subTaskCount > 0) {
            task.setTaskStatus(Task.TASK_STATUS_PENDING);
            task.setRunStatus(Task.RUN_STATUS_RUNNABLE);
        } else {
            task.setTaskStatus(Task.TASK_STATUS_FINISH);
            task.setRunStatus(Task.RUN_STATUS_STATUS_RUNNING);
        }

        taskService.updateById(task);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        taskStateContext.putResource(Task.TASK_STATUS_PENDING, applicationContext.getBean(this.getClass()));
    }
}
