package com.ruoyi.web;

import java.util.Arrays;
import java.util.List;

import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.domain.vo.PhotoStorageVO;
import com.ruoyi.service.UserPhotoStorageService;
import com.ruoyi.web.request.UserPhotoStorageCreateRequest;
import com.ruoyi.web.request.UserPhotoStorageEditRequest;
import com.ruoyi.web.request.UserPhotoStorageListRequest;
import com.ruoyi.web.request.UserPhotoStorageRemoveRequest;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.domain.UserPhotoStorage;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 用户照片存储Controller
 *
 * @author gxk
 * @date 2024-01-29
 */
@Api(tags = "用户照片存储")
@RestController
@RequestMapping("/review/storage")
public class UserPhotoStorageController extends BaseController
{

    @Autowired
    private UserPhotoStorageService userPhotoStorageService;

    /**
     * 查询用户照片存储列表
     */
    @ApiOperation("分页查询照片存储列表")
    @PostMapping("/list")
    public TableDataInfo<PhotoStorageVO> list(@Validated @RequestBody UserPhotoStorageListRequest request)
    {
//        startPage();
        List<PhotoStorageVO> list = userPhotoStorageService.selectUserPhotoStorageList(request);
        return getDataTable(list);
    }

    /**
     * 新增用户照片存储
     */
    @ApiOperation("新增用户照片存储")
    @Log(title = "用户照片存储", businessType = BusinessType.INSERT)
    @PostMapping("/create")
    public R<Integer> create(@Validated @RequestBody UserPhotoStorageCreateRequest request)
    {
        return R.ok(userPhotoStorageService.insertUserPhotoStorage(request));
    }

    /**
     * 修改用户照片存储
     */
    @ApiOperation("修改用户照片存储")
    @Log(title = "用户照片存储", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    public R<Boolean> edit(@Validated @RequestBody UserPhotoStorageEditRequest request)
    {
        UserPhotoStorage userPhotoStorage = UserPhotoStorage.builder()
                .id(request.getId())
                .path(request.getPath()).build();
        return R.ok(userPhotoStorageService.updateById(userPhotoStorage));
    }

    /**
     * 删除用户照片存储
     */
    @ApiOperation("批量删除用户照片存储")
    @Log(title = "用户照片存储", businessType = BusinessType.DELETE)
	@PostMapping("/remove")
    public R<Boolean> remove(@Validated @RequestBody UserPhotoStorageRemoveRequest request)
    {
        return R.ok(userPhotoStorageService.removeBatchByIds(Arrays.asList(request.getIds())));
    }
}
